;**********************************************************************
; TITLE : Multiplexer data DECODER
;**********************************************************************
;	Filename	: MUX_DE10.ASM
;    	Date Started 	: 21/7/98
;	Last edit 	: 31/7/98
;    	File Version	: 1.0
;                                                                     
;	Author		: David L. Jones
;**********************************************************************
;                                                                     
; Files required:
;	p16F84.inc	
;                                                                     
;**********************************************************************
; Notes:            
;	ASSUMING 1MHz XTAL CLOCK WHICH GIVES 4us PER INSTRUCTION
;	
;**********************************************************************

	list      p=16F84             ; list directive to define processor
	#include <p16F84.inc>         ; processor specific variable definitions

	__CONFIG   _CP_OFF & _WDT_ON & _PWRTE_ON & _XT_OSC

;***** VARIABLE DEFINITIONS
	
LOOP1	EQU	0CH		; LOOP VARIABLE FOR 1ms ROUTINE
COUNT1	EQU	0DH		; DELAY VARIABLE
COUNT2	EQU	0EH		; DELAY VARIABLE
OUTP	EQU	0FH		; TEMP OUTPUT VARIABLE FOR DATA LATCHING
	
;**********************************************************************
	ORG     0x000		; processor reset vector - start of code
	
;******************************* INITIALISATION STUFF
	CLRF	PORTB		; reset portb
	BSF	STATUS, RP0	; select bank 1
	MOVLW	B'00010000'	; PINS RB4=INPUT & ALL OTHERS=OUTPUT
 	MOVWF	TRISB		; UPDATE TRISB REGISTER TO DEFINE I/O PINS
	BCF	STATUS, RP0	; SELECT BANK 0
;******************************* 
START:
	CLRWDT			; RESET WATCHDOG TIMER
	CLRF	COUNT1		; CLEAR LOWER COUNT VARIABLE
	CLRF	COUNT2		; CLEAR UPPER COUNT VARIABLE
CLOOP:
	INCFSZ	COUNT1,1	; INC LOW COUNTER AND SKIP NEXT STATEMENT IF ZERO
	GOTO	SKIP1
	INCF	COUNT2,1	; INCREMENT THE HIGH COUNT VARIABLE
SKIP1:
	BTFSS	PORTB,4		; IF DATA INPUT IF HIGH THEN SKIP NEXT STATEMENT
	GOTO	CLOOP		; LOOP WHILE DATA INPUT IS LOW
	MOVLW	0F0H		
	ANDWF	COUNT2,0	; MASK OUT LOWER 4 BITS OF HIGH COUNTER (TO SET Z FLAG)
				; THIS CHECKS TO SEE IF AT LEAST 16ms HAVE PASSED
	BTFSC	STATUS,Z	; SKIP NEXT STATEMENT OF ZERO FLAG IF NOT SET
	GOTO	CLOOP		; CONTINUE TIMING AND CHECKING PORT IF NOT AT LEAST 16ms
	
	CLRF	OUTP		; CLEAR TEMP OUTPUT LATCH VARIABLE	

	CALL	DELAY1		; DELAY FOR 1ms TO WAIT FOR THE MIDDLE OF BIT0
	BTFSC	PORTB,4		; CHECK IF DATA INPUT IS HIGH
	BSF	OUTP,0		; SET BIT0

	CALL	DELAY1		; DELAY FOR 1ms TO WAIT FOR THE MIDDLE OF BIT1
	BTFSC	PORTB,4		; CHECK IF DATA INPUT IS HIGH
	BSF	OUTP,1		; SET BIT0

	CALL	DELAY1		; DELAY FOR 1ms TO WAIT FOR THE MIDDLE OF BIT2
	BTFSC	PORTB,4		; CHECK IF DATA INPUT IS HIGH
	BSF	OUTP,2		; SET BIT0

	CALL	DELAY1		; DELAY FOR 1ms TO WAIT FOR THE MIDDLE OF BIT3
	BTFSC	PORTB,4		; CHECK IF DATA INPUT IS HIGH
	BSF	OUTP,3		; SET BIT0

	MOVF	OUTP,0		; MOVE LATCH VARIABLE INTO W
	MOVWF	PORTB		; LATCH BIT0-BIT3 ONTO PORT PINS

	GOTO	START		; CONTINUE LOOPING FOREVER

;******************************** 1ms DELAY SUBROUTINE
DELAY1:
	MOVLW	0x52		; LOOP COUNT VALUE
	MOVWF	LOOP1		; SET LOOP VARIABLE
L1:	DECFSZ	LOOP1,1		; DEC LOOP COUNTER AND SKIP NEXT STATEMENT IF LOOP COUNTER=0
	GOTO 	L1		; CONTINUE LOOPING UNTIL ZERO
	RETURN

	END			; END OF PROGRAM
